/**************************************************************************
**       Title: image
**    $RCSfile: Image.hh,v $
**   $Revision: 1.1 $$Name:  $
**       $Date: 2005/05/02 12:07:11 $
**   Copyright: GPL $Author: fehr $
** Description:
**
**    
**
**-------------------------------------------------------------------------
**
**  $Log: Image.hh,v $
**
**
**************************************************************************/

#ifndef IMAGE_HH
#define IMAGE_HH

#include <fstream>
#include <iostream>
#include "MyError.hh"
#include "RGBColor.hh"
#include <cctype>
#include <iomanip>

/*======================================================================*/
/*!
 *  \class Image  Image.hh
 *  \brief The Image class is a very basic image class for
 *  storing and manipulating Int images.
 *
 */
/*======================================================================*/
template <typename T>
class Image
{

    static const unsigned int MAX_DIMENSION = 16384;
    
public:
  /*======================================================================*/
  /*! 
   *   Default Constructor. Creates an empty image
   */
  /*======================================================================*/
  Image();

  /*======================================================================*/
  /*! 
   *   Copy Constructor.
   */
  /*======================================================================*/
      
  Image(const Image<T>& image);
  
  /*======================================================================*/
  /*! 
   *   Size Constructor. Creates an image of arbitrary dimensions
   *
   *   \param width   width of image
   *   \param height  height of image
   */
  /*======================================================================*/
  Image(unsigned int width, unsigned int height);


  /*======================================================================*/
  /*! 
   *   Destructor
   */
  /*======================================================================*/
  ~Image();
  
  /*======================================================================*/
  /*! 
   *   resize image to width*height. If it has already this size, nothing
   *   is done 
   *
   *   \param width   width of image
   *   \param height  height of image
   */
  /*======================================================================*/
  void resize(unsigned int width, unsigned int height);
  

  /*======================================================================*/
  /*! 
   *   unchecked access to a pixel (two-dimensional position)
   *
   *   \param x    horizontal pixel coordinate
   *   \param y    vertical pixel coordinate
   *
   *   \return  reference to the pixel at position row,col
   */
  /*======================================================================*/
  T&       operator()(unsigned int x, unsigned int y);
  const T& operator()(unsigned int x, unsigned int y) const;
  


  /*======================================================================*/
  /*! 
   *   width of the image
   *
   *   \return image width
   */
  /*======================================================================*/
  unsigned int width() const;

  
  /*======================================================================*/
  /*! 
   *   height of the image
   *
   *   \return image height
   */
  /*======================================================================*/
  unsigned int height() const;

  /*======================================================================*/
  /*! 
   *   size of the image (number of pixels)
   *
   *   \return number of pixels
   */
  /*======================================================================*/
  size_t size() const;

  /*======================================================================*/
  /*! 
   *   begin of image
   *
   *   \return pointer to first RGBColor of image
   */
  /*======================================================================*/
  T*       begin();
  const T* begin() const;

  /*======================================================================*/
  /*! 
   *   end of image
   *
   *   \return pointer to the first RGBColor BEHIND the last RGBColor 
   *           of the image
   */
  /*======================================================================*/
  T*       end();
  const T* end() const;

  /*======================================================================*/
  /*! 
   *   operator[] 
   *
   *   \param n    pixel number
   *   \return reference to RGBColor number n of image
   */
  /*======================================================================*/
  T&       operator[](size_t n); 
  const T& operator[](size_t n) const;


  /*======================================================================*/
  /*! 
   *   operator=
   *
   *   \param image    image to copy
   *   \return reference to modified image
   */
  /*======================================================================*/
  Image<T>& operator=(const Image<T>& image);

  /*======================================================================*/
  /*!
   *   read an image from the given stream in ASCII PPM format (image
   *   Header "P3")
   *
   *   \param is   input stream. Must be already opened for reading,
   *               with the read postition pointing to the begin of
   *               the image header
   *
   *   \exception ReadError an error occured during
   *                        reading. ReadError::msg() contains the plain
   *                        text error message.
   */
  /*======================================================================*/
  void read(std::istream& is);   
  
  /*======================================================================*/
  /*!
   *   write the image to the given stream in ASCII PPM format (image
   *   Header "P3")
   *
   *   \param os  output stream. Must be already opened for writing
   *
   *   \exception WriteError an error occured during
   *                         writing. WriteError::msg() contains the plain
   *                         text error message.
   */
  /*======================================================================*/ 
  void write(std::ostream& os);  
 
private:
  unsigned int _width, _height, _size;
  T* _pixels;
  T* _pixelsEnd;
  T** _rowStart;
};


  /*======================================================================*/
  /*!
   *   specialization of read for RGBColor
   *   read an image from the given stream in ASCII PPM format (image
   *   Header "P3")
   *
   *   \param is   input stream. Must be already opened for reading,
   *               with the read postition pointing to the begin of
   *               the image header
   *
   *   \exception ReadError an error occured during
   *                        reading. ReadError::msg() contains the plain
   *                        text error message.
   */
  /*======================================================================*/
   template<>
   void Image<RGBColor>::read(std::istream& is);

  /*======================================================================*/
  /*!
   *   specialization of write for RGBColor 
   *   write the image to the given stream in ASCII PPM format (image
   *   Header "P3")
   *
   *   \param os  output stream. Must be already opened for writing
   *
   *   \exception WriteError an error occured during
   *                         writing. WriteError::msg() contains the plain
   *                         text error message.
   */
  /*======================================================================*/
   template<>
   void Image<RGBColor>::write(std::ostream& is);


#include "Image.icc"

#endif
